@@ -1,9 +1,7 @@
 package SQL::Translator::Parser::PostgreSQL;
 
 # -------------------------------------------------------------------
-# $Id: PostgreSQL.pm,v 1.48 2006/08/26 11:34:49 schiffbruechige Exp $
-# -------------------------------------------------------------------
-# Copyright (C) 2002-4 SQLFairy Authors
+# Copyright (C) 2002-2009 SQLFairy Authors
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License as
@@ -108,7 +106,7 @@ View table:
 
 use strict;
 use vars qw[ $DEBUG $VERSION $GRAMMAR @EXPORT_OK ];
-$VERSION = sprintf "%d.%02d", q$Revision: 1.48 $ =~ /(\d+)\.(\d+)/;
+$VERSION = '1.59';
 $DEBUG   = 0 unless defined $DEBUG;
 
 use Data::Dumper;
@@ -128,7 +126,7 @@ my $parser; # should we do this?  There's no programmic way to
 
 $GRAMMAR = q!
 
-{ my ( %tables, $table_order, $field_order, @table_comments) }
+{ my ( %tables, @views, $table_order, $field_order, @table_comments) }
 
 #
 # The "eofile" rule makes the parser fail if any "statement" rule
@@ -136,7 +134,7 @@ $GRAMMAR = q!
 # won't cause the failure needed to know that the parse, as a whole,
 # failed. -ky
 #
-startrule : statement(s) eofile { \%tables }
+startrule : statement(s) eofile { { tables => \%tables, views => \@views } }
 
 eofile : /^\Z/
    
@@ -154,15 +152,20 @@ statement : create
   | connect
   | update
   | set
+  | select
+  | copy
+  | readin_symbol
   | <error>
 
 connect : /^\s*\\\connect.*\n/
 
 set : /set/i /[^;]*/ ';'
 
-revoke : /revoke/i WORD(s /,/) /on/i TABLE(?) table_name /from/i name_with_opt_quotes(s /,/) ';'
+revoke : /revoke/i WORD(s /,/) /on/i TABLE(?) table_id /from/i name_with_opt_quotes(s /,/) ';'
     {
-        my $table_name = $item{'table_name'};
+        my $table_info  = $item{'table_id'};
+        my $schema_name = $table_info->{'schema_name'};
+        my $table_name  = $table_info->{'table_name'};
         push @{ $tables{ $table_name }{'permissions'} }, {
             type       => 'revoke',
             actions    => $item[2],
@@ -170,9 +173,14 @@ revoke : /revoke/i WORD(s /,/) /on/i TABLE(?) table_name /from/i name_with_opt_q
         }
     }
 
-grant : /grant/i WORD(s /,/) /on/i TABLE(?) table_name /to/i name_with_opt_quotes(s /,/) ';'
+revoke : /revoke/i WORD(s /,/) /on/i SCHEMA(?) schema_name /from/i name_with_opt_quotes(s /,/) ';'
+    { 1 }
+
+grant : /grant/i WORD(s /,/) /on/i TABLE(?) table_id /to/i name_with_opt_quotes(s /,/) ';'
     {
-        my $table_name = $item{'table_name'};
+        my $table_info  = $item{'table_id'};
+        my $schema_name = $table_info->{'schema_name'};
+        my $table_name  = $table_info->{'table_name'};
         push @{ $tables{ $table_name }{'permissions'} }, {
             type       => 'grant',
             actions    => $item[2],
@@ -180,6 +188,9 @@ grant : /grant/i WORD(s /,/) /on/i TABLE(?) table_name /to/i name_with_opt_quote
         }
     }
 
+grant : /grant/i WORD(s /,/) /on/i SCHEMA(?) schema_name /to/i name_with_opt_quotes(s /,/) ';'
+    { 1 }
+
 drop : /drop/i /[^;]*/ ';'
 
 string :
@@ -187,20 +198,25 @@ string :
 
 nonstring : /[^;\'"]+/
 
-statement_body : (string | nonstring)(s?)
+statement_body : string | nonstring
 
-insert : /insert/i statement_body ';'
+insert : /insert/i statement_body(s?) ';'
 
-update : /update/i statement_body ';'
+update : /update/i statement_body(s?) ';'
 
 #
 # Create table.
 #
-create : create_table table_name '(' create_definition(s? /,/) ')' table_option(s?) ';'
+create : CREATE temporary(?) TABLE table_id '(' create_definition(s? /,/) ')' table_option(s?) ';'
     {
-        my $table_name                       = $item{'table_name'};
-        $tables{ $table_name }{'order'}      = ++$table_order;
-        $tables{ $table_name }{'table_name'} = $table_name;
+        my $table_info  = $item{'table_id'};
+        my $schema_name = $table_info->{'schema_name'};
+        my $table_name  = $table_info->{'table_name'};
+        $tables{ $table_name }{'order'}       = ++$table_order;
+        $tables{ $table_name }{'schema_name'} = $schema_name;
+        $tables{ $table_name }{'table_name'}  = $table_name;
+
+        $tables{ $table_name }{'temporary'} = $item[2][0];
 
         if ( @table_comments ) {
             $tables{ $table_name }{'comments'} = [ @table_comments ];
@@ -208,7 +224,7 @@ create : create_table table_name '(' create_definition(s? /,/) ')' table_option(
         }
 
         my @constraints;
-        for my $definition ( @{ $item[4] } ) {
+        for my $definition ( @{ $item[6] } ) {
             if ( $definition->{'supertype'} eq 'field' ) {
                 my $field_name = $definition->{'name'};
                 $tables{ $table_name }{'fields'}{ $field_name } = 
@@ -228,7 +244,7 @@ create : create_table table_name '(' create_definition(s? /,/) ')' table_option(
             }
         }
 
-        for my $option ( @{ $item[6] } ) {
+        for my $option ( @{ $item[8] } ) {
             $tables{ $table_name }{'table_options(s?)'}{ $option->{'type'} } = 
                 $option;
         }
@@ -236,9 +252,12 @@ create : create_table table_name '(' create_definition(s? /,/) ')' table_option(
         1;
     }
 
-create : CREATE unique(?) /(index|key)/i index_name /on/i table_name using_method(?) '(' field_name(s /,/) ')' where_predicate(?) ';'
+create : CREATE unique(?) /(index|key)/i index_name /on/i table_id using_method(?) '(' field_name(s /,/) ')' where_predicate(?) ';'
     {
-        push @{ $tables{ $item{'table_name'} }{'indices'} },
+        my $table_info  = $item{'table_id'};
+        my $schema_name = $table_info->{'schema_name'};
+        my $table_name  = $table_info->{'table_name'};
+        push @{ $tables{ $table_name }{'indices'} },
             {
                 name      => $item{'index_name'},
                 supertype => $item{'unique'}[0] ? 'constraint' : 'index',
@@ -247,7 +266,17 @@ create : CREATE unique(?) /(index|key)/i index_name /on/i table_name using_metho
                 method    => $item{'using_method'}[0],
             }
         ;
+    }
 
+create : CREATE or_replace(?) temporary(?) VIEW view_id view_fields(?) /AS/i view_target ';'
+    {
+        push @views, {
+            schema_name  => $item{view_id}{schema_name},
+            view_name    => $item{view_id}{view_name},
+            sql          => $item{view_target},
+            fields       => $item[6],
+            is_temporary => $item[3][0],
+        }
     }
 
 #
@@ -273,9 +302,12 @@ comment : /^\s*(?:#|-{2})(.*)\n/
         push @table_comments, $comment;
     }
 
-comment_on_table : /comment/i /on/i /table/i table_name /is/i comment_phrase ';'
+comment_on_table : /comment/i /on/i /table/i table_id /is/i comment_phrase ';'
     {
-        push @{ $tables{ $item{'table_name'} }{'comments'} }, $item{'comment_phrase'};
+        my $table_info  = $item{'table_id'};
+        my $schema_name = $table_info->{'schema_name'};
+        my $table_name  = $table_info->{'table_name'};
+        push @{ $tables{ $table_name }{'comments'} }, $item{'comment_phrase'};
     }
 
 comment_on_column : /comment/i /on/i /column/i column_name /is/i comment_phrase ';'
@@ -384,6 +416,9 @@ field_comment : /^\s*(?:#|-{2})(.*)\n/
 field_meta : default_val
     | column_constraint
 
+view_fields : '(' field_name(s /,/) ')'
+    { $return = join (',', @{$item[2]} ) }
+
 column_constraint : constraint_name(?) column_constraint_type deferrable(?) deferred(?)
     {
         my $desc       = $item{'column_constraint_type'};
@@ -422,8 +457,11 @@ column_constraint_type : /not null/i { $return = { type => 'not_null' } }
     /check/i '(' /[^)]+/ ')' 
         { $return = { type => 'check', expression => $item[3] } }
     |
-    /references/i table_name parens_word_list(?) match_type(?) key_action(s?)
+    /references/i table_id parens_word_list(?) match_type(?) key_action(s?)
     {
+        my $table_info  = $item{'table_id'};
+        my $schema_name = $table_info->{'schema_name'};
+        my $table_name  = $table_info->{'table_name'};
         my ( $on_delete, $on_update );
         for my $action ( @{ $item[5] || [] } ) {
             $on_delete = $action->{'action'} if $action->{'type'} eq 'delete';
@@ -432,7 +470,7 @@ column_constraint_type : /not null/i { $return = { type => 'not_null' } }
 
         $return              =  {
             type             => 'foreign_key',
-            reference_table  => $item[2],
+            reference_table  => $table_name,
             reference_fields => $item[3][0],
             match_type       => $item[4][0],
             on_delete        => $on_delete,
@@ -440,7 +478,29 @@ column_constraint_type : /not null/i { $return = { type => 'not_null' } }
         }
     }
 
-table_name : name_with_opt_quotes
+table_id : schema_qualification(?) name_with_opt_quotes {
+    $return = { schema_name => $item[1][0], table_name => $item[2] }
+}
+
+view_id : schema_qualification(?) name_with_opt_quotes {
+    $return = { schema_name => $item[1][0], view_name => $item[2] }
+}
+
+view_target : /select|with/i /[^;]+/ {
+    $return = "$item[1] $item[2]";
+}
+
+# SELECT views _may_ support outer parens, and we used to produce
+# such sql, although non-standard. Use ugly lookeahead to parse
+view_target : '('   /select/i    / [^;]+ (?= \) ) /x    ')'    {
+    $return = "$item[2] $item[3]"
+}
+
+view_target_spec :  
+
+schema_qualification : name_with_opt_quotes '.'
+
+schema_name : name_with_opt_quotes
 
 field_name : name_with_opt_quotes
 
@@ -448,7 +508,7 @@ name_with_opt_quotes : double_quote(?) NAME double_quote(?) { $item[2] }
 
 double_quote: /"/
 
-index_name : WORD
+index_name : name_with_opt_quotes
 
 data_type : pg_data_type parens_value_list(?)
     { 
@@ -574,7 +634,8 @@ pg_data_type :
 parens_value_list : '(' VALUE(s /,/) ')'
     { $item[2] }
 
-parens_word_list : '(' WORD(s /,/) ')'
+
+parens_word_list : '(' name_with_opt_quotes(s /,/) ')'
     { $item[2] }
 
 field_size : '(' num_range ')' { $item{'num_range'} }
@@ -593,7 +654,7 @@ table_constraint : comment(s?) constraint_name(?) table_constraint_type deferrab
         my @comments   = ( @{ $item[1] }, @{ $item[-1] } );
 
         $return              =  {
-            name             => $item{'constraint_name'}[0] || '',
+            name             => $item[2][0] || '',
             supertype        => 'constraint',
             type             => $type,
             fields           => $type ne 'check' ? $fields : [],
@@ -633,7 +694,7 @@ table_constraint_type : /primary key/i '(' name_with_opt_quotes(s /,/) ')'
         }
     }
     |
-    /foreign key/i '(' name_with_opt_quotes(s /,/) ')' /references/i table_name parens_word_list(?) match_type(?) key_action(s?)
+    /foreign key/i '(' name_with_opt_quotes(s /,/) ')' /references/i table_id parens_word_list(?) match_type(?) key_action(s?)
     {
         my ( $on_delete, $on_update );
         for my $action ( @{ $item[9] || [] } ) {
@@ -645,7 +706,7 @@ table_constraint_type : /primary key/i '(' name_with_opt_quotes(s /,/) ')'
             supertype        => 'constraint',
             type             => 'foreign_key',
             fields           => $item[3],
-            reference_table  => $item[6],
+            reference_table  => $item[6]->{'table_name'},
             reference_fields => $item[7][0],
             match_type       => $item[8][0],
             on_delete     => $on_delete || '',
@@ -653,7 +714,7 @@ table_constraint_type : /primary key/i '(' name_with_opt_quotes(s /,/) ')'
         }
     }
 
-deferrable : /not/i /deferrable/i 
+deferrable : not(?) /deferrable/i 
     { 
         $return = ( $item[1] =~ /not/i ) ? 0 : 1;
     }
@@ -694,32 +755,32 @@ key_mutation : /no action/i { $return = 'no_action' }
     |
     /set default/i { $return = 'set default' }
 
-alter : alter_table table_name add_column field ';' 
+alter : alter_table table_id add_column field ';' 
     { 
         my $field_def = $item[4];
-        $tables{ $item[2] }{'fields'}{ $field_def->{'name'} } = {
+        $tables{ $item[2]->{'table_name'} }{'fields'}{ $field_def->{'name'} } = {
             %$field_def, order => $field_order++
         };
         1;
     }
 
-alter : alter_table table_name ADD table_constraint ';' 
+alter : alter_table table_id ADD table_constraint ';' 
     { 
-        my $table_name = $item[2];
+        my $table_name = $item[2]->{'table_name'};
         my $constraint = $item[4];
         push @{ $tables{ $table_name }{'constraints'} }, $constraint;
         1;
     }
 
-alter : alter_table table_name drop_column NAME restrict_or_cascade(?) ';' 
+alter : alter_table table_id drop_column NAME restrict_or_cascade(?) ';' 
     {
-        $tables{ $item[2] }{'fields'}{ $item[4] }{'drop'} = 1;
+        $tables{ $item[2]->{'table_name'} }{'fields'}{ $item[4] }{'drop'} = 1;
         1;
     }
 
-alter : alter_table table_name alter_column NAME alter_default_val ';' 
+alter : alter_table table_id alter_column NAME alter_default_val ';' 
     {
-        $tables{ $item[2] }{'fields'}{ $item[4] }{'default'} = 
+        $tables{ $item[2]->{'table_name'} }{'fields'}{ $item[4] }{'default'} = 
             $item[5]->{'value'};
         1;
     }
@@ -727,26 +788,36 @@ alter : alter_table table_name alter_column NAME alter_default_val ';'
 #
 # These will just parse for now but won't affect the structure. - ky
 #
-alter : alter_table table_name /rename/i /to/i NAME ';'
+alter : alter_table table_id /rename/i /to/i NAME ';'
     { 1 }
 
-alter : alter_table table_name alter_column NAME SET /statistics/i INTEGER ';' 
+alter : alter_table table_id alter_column NAME SET /statistics/i INTEGER ';' 
     { 1 }
 
-alter : alter_table table_name alter_column NAME SET /storage/i storage_type ';'
+alter : alter_table table_id alter_column NAME SET /storage/i storage_type ';'
     { 1 }
 
-alter : alter_table table_name rename_column NAME /to/i NAME ';'
+alter : alter_table table_id rename_column NAME /to/i NAME ';'
     { 1 }
 
-alter : alter_table table_name DROP /constraint/i NAME restrict_or_cascade ';'
+alter : alter_table table_id DROP /constraint/i NAME restrict_or_cascade ';'
     { 1 }
 
-alter : alter_table table_name /owner/i /to/i NAME ';'
+alter : alter_table table_id /owner/i /to/i NAME ';'
+    { 1 }
+
+alter : alter_sequence NAME /owned/i /by/i column_name ';'
     { 1 }
 
 storage_type : /(plain|external|extended|main)/i
 
+temporary : /temp(orary)?\\b/i
+  {
+    1;
+  }
+
+or_replace : /or replace/i
+
 alter_default_val : SET default_val 
     { 
         $return = { value => $item[2]->{'value'} } 
@@ -763,9 +834,9 @@ alter_default_val : SET default_val
 # particular order.  I'm going to leave the rule but disable the code 
 # for now. - ky
 #
-alter : alter_table table_name alter_column NAME alter_nullable ';'
+alter : alter_table table_id alter_column NAME alter_nullable ';'
     {
-#        my $table_name  = $item[2];
+#        my $table_name  = $item[2]->{'table_name'};
 #        my $field_name  = $item[4];
 #        my $is_nullable = $item[5]->{'is_nullable'};
 #
@@ -806,10 +877,14 @@ alter_nullable : SET not_null
 
 not_null : /not/i /null/i
 
+not : /not/i
+
 add_column : ADD COLUMN(?)
 
 alter_table : ALTER TABLE ONLY(?)
 
+alter_sequence : ALTER SEQUENCE 
+
 drop_column : DROP COLUMN(?)
 
 alter_column : ALTER COLUMN(?)
@@ -819,6 +894,24 @@ rename_column : /rename/i COLUMN(?)
 restrict_or_cascade : /restrict/i | 
     /cascade/i
 
+# Handle functions that can be called
+select : SELECT select_function ';' 
+    { 1 }
+
+# Read the setval function but don't do anything with it because this parser
+# isn't handling sequences
+select_function : schema_qualification(?) /setval/i '(' VALUE /,/ VALUE /,/ /(true|false)/i ')' 
+    { 1 }
+
+# Skipping all COPY commands
+copy : COPY WORD /[^;]+/ ';' { 1 }
+    { 1 }
+
+# The "\." allows reading in from STDIN but this isn't needed for schema
+# creation, so it is skipped.
+readin_symbol : '\.'
+    {1}
+
 #
 # End basically useless stuff. - ky
 #
@@ -827,7 +920,7 @@ create_table : CREATE TABLE
 
 create_index : CREATE /index/i
 
-default_val  : DEFAULT /(\d+|'[^']*'|\w+\(.*?\))|\w+/
+default_val  : DEFAULT /(\d+|'[^']*'|\w+\(.*\))|\w+/
     { 
         my $val =  defined $item[2] ? $item[2] : '';
         $val    =~ s/^'|'$//g; 
@@ -879,8 +972,18 @@ COLUMN : /column/i
 
 TABLE : /table/i
 
+VIEW : /view/i
+
+SCHEMA : /schema/i
+
 SEMICOLON : /\s*;\n?/
 
+SEQUENCE : /sequence/i
+
+SELECT : /select/i
+
+COPY : /copy/i
+
 INTEGER : /\d+/
 
 WORD : /\w+/
@@ -926,15 +1029,18 @@ sub parse {
 
     my $schema = $translator->schema;
     my @tables = sort { 
-        ( $result->{ $a }{'order'} || 0 ) <=> ( $result->{ $b }{'order'} || 0 )
-    } keys %{ $result };
+        ( $result->{tables}{ $a }{'order'} || 0 ) <=> ( $result->{tables}{ $b }{'order'} || 0 )
+    } keys %{ $result->{tables} };
 
     for my $table_name ( @tables ) {
-        my $tdata =  $result->{ $table_name };
+        my $tdata =  $result->{tables}{ $table_name };
         my $table =  $schema->add_table( 
-            name  => $tdata->{'table_name'},
+            #schema => $tdata->{'schema_name'},
+            name   => $tdata->{'table_name'},
         ) or die "Couldn't create table '$table_name': " . $schema->error;
 
+        $table->extra(temporary => 1) if $tdata->{'temporary'};
+
         $table->comments( $tdata->{'comments'} );
 
         my @fields = sort { 
@@ -970,7 +1076,7 @@ sub parse {
                 name   => $idata->{'name'},
                 type   => uc $idata->{'type'},
                 fields => $idata->{'fields'},
-            ) or die $table->error;
+            ) or die $table->error . ' ' . $table->name;
         }
 
         for my $cdata ( @{ $tdata->{'constraints'} || [] } ) {
@@ -990,6 +1096,18 @@ sub parse {
         }
     }
 
+    for my $vinfo (@{$result->{views}}) {
+      my $sql = $vinfo->{sql};
+      $sql =~ s/\A\s+|\s+\z//g;
+      my $view = $schema->add_view (
+        name => $vinfo->{view_name},
+        sql => $sql,
+        fields => $vinfo->{fields},
+      );
+
+      $view->extra ( temporary => 1 ) if $vinfo->{is_temporary};
+    }
+
     return 1;
 }
 
