@@ -1,7 +1,5 @@
 package SQL::Translator::Producer::XML::SQLFairy;
 
-# -------------------------------------------------------------------
-# $Id: SQLFairy.pm,v 1.20 2005/07/05 16:20:43 mwz444 Exp $
 # -------------------------------------------------------------------
 # Copyright (C) 2003 Ken Y. Clark <kclark@cpan.org>,
 #                    darren chamberlain <darren@cpan.org>,
@@ -165,7 +163,7 @@ To convert your old format files simply pass them through the translator :)
 
 use strict;
 use vars qw[ $VERSION @EXPORT_OK ];
-$VERSION = sprintf "%d.%02d", q$Revision: 1.20 $ =~ /(\d+)\.(\d+)/;
+$VERSION = '1.59';
 
 use Exporter;
 use base qw(Exporter);
@@ -189,10 +187,12 @@ my @MAP_AS_ELEMENTS = qw/sql comments action extra/;
 my $Namespace = 'http://sqlfairy.sourceforge.net/sqlfairy.xml';
 my $Name      = 'sqlf';
 my $PArgs     = {};
+my $no_comments;
 
 sub produce {
     my $translator  = shift;
     my $schema      = $translator->schema;
+    $no_comments    = $translator->no_comments;
     $PArgs          = $translator->producer_args;
     my $newlines    = defined $PArgs->{newlines} ? $PArgs->{newlines} : 1;
     my $indent      = defined $PArgs->{indent}   ? $PArgs->{indent}   : 2;
@@ -212,7 +212,10 @@ sub produce {
 
     # Start the document
     $xml->xmlDecl('UTF-8');
-    $xml->comment(header_comment('', ''));
+
+    $xml->comment(header_comment('', ''))
+      unless $no_comments;
+
     xml_obj($xml, $schema,
         tag => "schema", methods => [qw/name database extra/], end_tag => 0 );
 
@@ -288,7 +291,7 @@ sub produce {
     #
     xml_obj_children( $xml, $schema,
         tag    => 'trigger',
-        methods => [qw/name database_event action on_table perform_action_when
+        methods => [qw/name database_events action on_table perform_action_when
             fields order extra/],
     );
 
