@@ -1,9 +1,7 @@
 package SQL::Translator::Schema::Constants;
 
 # ----------------------------------------------------------------------
-# $Id: Constants.pm,v 1.43 2004/02/09 22:15:15 kycl4rk Exp $
-# ----------------------------------------------------------------------
-# Copyright (C) 2002-4 SQLFairy Authors
+# Copyright (C) 2002-2009 SQLFairy Authors
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License as
@@ -63,12 +61,13 @@ use strict;
 use base qw( Exporter );
 use vars qw( @EXPORT $VERSION );
 require Exporter;
-$VERSION = sprintf "%d.%02d", q$Revision: 1.43 $ =~ /(\d+)\.(\d+)/;
+$VERSION = '1.59';
 
 @EXPORT = qw[ 
     CHECK_C
     FOREIGN_KEY
     FULL_TEXT
+    SPATIAL
     NOT_NULL
     NORMAL
     NULL
@@ -85,6 +84,8 @@ use constant FOREIGN_KEY => 'FOREIGN KEY';
 
 use constant FULL_TEXT => 'FULLTEXT';
 
+use constant SPATIAL => 'SPATIAL';
+
 use constant NOT_NULL => 'NOT NULL';
 
 use constant NORMAL => 'NORMAL';
